@@ -171,8 +171,18 @@ gfarm_auth_request_sharedsecret(struct gfp_xdr *conn,
 			    gfarm_error_string(GFARM_ERR_UNEXPECTED_EOF));
 			return (GFARM_ERR_UNEXPECTED_EOF);
 		}
-		gfarm_auth_sharedsecret_response_data(shared_key, challenge,
-		    response);
+		e = gfarm_auth_sharedsecret_response_data(
+		    shared_key, challenge, response);
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1004515,
+			    "calculating challenge-response: %s",
+			    gfarm_error_string(e));
+			/*
+			 * this response must fail, we just want to prevent
+			 * uninitialized memory access here.
+			 */
+			memset(response, 0, sizeof response);
+		}
 		e = gfp_xdr_send(conn, "ib",
 		    expire, sizeof(response), response);
 		if (e == GFARM_ERR_NO_ERROR)
@@ -551,6 +561,7 @@ static void
 gfarm_auth_request_sharedsecret_receive_challenge(int events, int fd,
 	void *closure, const struct timeval *t)
 {
+	gfarm_error_t e;
 	struct gfarm_auth_request_sharedsecret_state *state = closure;
 	int rv, eof;
 	char challenge[GFARM_AUTH_CHALLENGE_LEN];
@@ -575,8 +586,18 @@ gfarm_auth_request_sharedsecret_receive_challenge(int events, int fd,
 		state->error = GFARM_ERR_UNEXPECTED_EOF;
 	if (state->error == GFARM_ERR_NO_ERROR) {
 		/* XXX It's better to check writable event here */
-		gfarm_auth_sharedsecret_response_data(
+		e = gfarm_auth_sharedsecret_response_data(
 		    state->shared_key, challenge, response);
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1004516,
+			    "calculating challenge-response: %s",
+			    gfarm_error_string(e));
+			/*
+			 * this response must fail, we just want to prevent
+			 * uninitialized memory access here.
+			 */
+			memset(response, 0, sizeof response);
+		}
 		state->error = gfp_xdr_send(state->conn, "ib",
 		    state->expire, sizeof(response), response);
 		if (state->error == GFARM_ERR_NO_ERROR &&
