@@ -263,7 +263,7 @@ enum gfm_proto_command {
 	GFM_PROTO_PROCESS_ALLOC_CHILD,
 	GFM_PROTO_PROCESS_FREE,
 	GFM_PROTO_PROCESS_SET,
-	GFM_PROTO_PROCESS_RESERVE4,
+	GFM_PROTO_PROCESS_FD_INFO,		/* since gfarm-2.6.14 */
 	GFM_PROTO_PROCESS_RESERVE5,
 	GFM_PROTO_PROCESS_RESERVE6,
 	GFM_PROTO_PROCESS_RESERVE7,
@@ -459,6 +459,53 @@ enum gfm_proto_command {
 #define GFM_PROTO_REPLICA_FLAG_DEAD_HOST	2
 #define GFM_PROTO_REPLICA_FLAG_DEAD_COPY	4
 
+/* flags of GFM_PROTO_PROCESS_FD_INFO */
+/*							0x00001 - reserved */
+/*							0x00002 - reserved */
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_WRITE_NO_OPEN	0x00004
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_WRITE_OPEN	0x00008
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_READ_NO_OPEN	0x00010
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_READ_OPEN	0x00020
+/*					LOOKUP_NO_OPEN	0x00040 - reserved */
+/*					LOOKUP_OPEN	0x00080 - reserved */
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_CLIENT_DETACH	0x00100
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_CLIENT_ATTACH	0x00200
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_GFSD_DETACH	0x00400
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_GFSD_ATTACH	0x00800
+/*	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_FIFO	0x01000 - reserved */
+/*	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_CHR	0x02000 - reserved */
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_DIR	0x04000
+/*	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_BLK	0x08000 - reserved */
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_REG	0x10000
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_LNK	0x20000
+/*	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_SOCK	0x40000 - reserved */
+/*	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_???	0x80000 - reserved */
+#define GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_ALLMASK	0xff000
+
+#define GFM_PROTO_PROCESS_FD_FLAG_MASK \
+	(gfarm_uint64_t)( \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_WRITE_OPEN| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_WRITE_NO_OPEN| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_READ_OPEN| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_READ_NO_OPEN| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_CLIENT_ATTACH| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_CLIENT_DETACH| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_GFSD_ATTACH| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_GFSD_DETACH| \
+	GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_INODE_ALLMASK)
+
+/*
+ * NOTE about GFM_PROTO_PROCESS_FD_INFO:
+ * - GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_GFSD_DETACH only affects INODE_REG,
+ *   i.e. other inode types aren't affected by this flag.
+ * - input parameter `gfsd_domain' only affects descriptors which are
+ *   GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_GFSD_ATTACH state.
+ * - input parameter `user_host_domain' only affects descriptors which are
+ *   GFM_PROTO_PROCESS_FD_FLAG_EXCLUDE_CLIENT_ATTACH state.
+ * these decisions were made to the search condition flexible.
+ */
+
+
 /* output of GFM_PROTO_METADB_SERVER_GET: Persistent Flags */
 #define GFARM_METADB_SERVER_FLAG_IS_MASTER_CANDIDATE	0x00000001
 #define GFARM_METADB_SERVER_FLAG_IS_DEFAULT_MASTER	0x00000002
