@@ -8,12 +8,11 @@
 #include <sys/stat.h>
 #include <ctype.h>
 #include <pwd.h>
+#include <gssapi.h>
 
 #include <gfarm/gflog.h>
 #include <gfarm/error.h>
 
-#include "gssapi.h"
-
 #include "gfevent.h"
 #include "gfutil.h"
 #include "thrsubr.h"
@@ -382,8 +381,6 @@ gfarmGssAcquireCredential(gss_cred_id_t *credPtr,
     gss_cred_id_t cred;
     static const char diag[] = "gfarmGssAcquireCredential";
     
-    *credPtr = GSS_C_NO_CREDENTIAL;
-
     gfarm_privilege_lock(diag);
     majStat = gss_acquire_cred(&minStat,
 			       desiredName,
@@ -592,11 +589,11 @@ gfarmGssAcceptSecurityContext(int fd, gss_cred_id_t cred, gss_ctx_id_t *scPtr,
     *scPtr = GSS_C_NO_CONTEXT;
 
     do {
-	tknStat = gfarmGssReceiveToken(fd, itPtr, GFARM_GSS_TIMEOUT_INFINITE);
+	tknStat = gfarmGssReceiveToken(fd, itPtr, GFARM_GSS_AUTH_TIMEOUT_MSEC);
 	if (tknStat <= 0) {
 	    gsiErrNo = errno;
 	    gflog_auth_info(GFARM_MSG_1003845,
-		"gfarmGssAcceptSecurityContext(): failed to receive response"
+		"gfarmGssAcceptSecurityContext(): failed to receive response: "
 		"%s", strerror(gsiErrNo));
 	    majStat = GSS_S_DEFECTIVE_TOKEN|GSS_S_CALL_INACCESSIBLE_READ;
 	    minStat = GFSL_DEFAULT_MINOR_ERROR;
@@ -630,7 +627,7 @@ gfarmGssAcceptSecurityContext(int fd, gss_cred_id_t cred, gss_ctx_id_t *scPtr,
 	    } else {
 		gflog_auth_info(GFARM_MSG_1003846,
 		    "gfarmGssAcceptSecurityContext(): failed to send response"
-		    "%s", strerror(gsiErrNo));
+		    ": %s", strerror(gsiErrNo));
 		majStat = GSS_S_DEFECTIVE_TOKEN|GSS_S_CALL_INACCESSIBLE_WRITE;
 		minStat = GFSL_DEFAULT_MINOR_ERROR;
 	    }
@@ -755,7 +752,7 @@ gfarmGssInitiateSecurityContext(int fd, const gss_name_t acceptorName,
     
 	if (majStat & GSS_S_CONTINUE_NEEDED) {
 	    tknStat = gfarmGssReceiveToken(fd, itPtr,
-					   GFARM_GSS_TIMEOUT_INFINITE);
+			GFARM_GSS_AUTH_TIMEOUT_MSEC);
 	    if (tknStat <= 0) {
 		gsiErrNo = errno;
 		gflog_auth_error(GFARM_MSG_1003848,
@@ -862,8 +859,9 @@ gfarmGssSend(int fd, gss_ctx_id_t sCtx, int doEncrypt, gss_qop_t qopReq,
 
     int sum = 0;
     int rem = n;
-    int len, tknStat;
+    int len, tknStat, save_errno;
     gfarm_int32_t n_buf = n;
+    static const char diag[] = "gfarmGssSend";
 
     /*
      * Send a length of a PLAIN TEXT.
@@ -879,6 +877,7 @@ gfarmGssSend(int fd, gss_ctx_id_t sCtx, int doEncrypt, gss_qop_t qopReq,
     }
 
     if (gfarmWriteInt32(fd, &n_buf, 1) != 1) {
+	gflog_info(GFARM_MSG_1004283, "%s: %s", diag, strerror(errno));
 	majStat = GSS_S_CALL_INACCESSIBLE_WRITE;
 	goto Done;
     }
@@ -894,18 +893,25 @@ gfarmGssSend(int fd, gss_ctx_id_t sCtx, int doEncrypt, gss_qop_t qopReq,
 	if (majStat == GSS_S_COMPLETE) {
 	    if (otPtr->length > 0) {
 		tknStat = gfarmGssSendToken(fd, otPtr);
+		save_errno = errno;
 		gss_release_buffer(&minStat, otPtr);
 		if (tknStat <= 0) {
+		    gflog_info(GFARM_MSG_1004284, "%s: %s", diag,
+			strerror(save_errno));
 		    majStat = GSS_S_DEFECTIVE_TOKEN|GSS_S_CALL_INACCESSIBLE_WRITE;
 		    goto Done;
 		}
 		rem -= len;
 		sum += len;
 	    } else {
+		gflog_info(GFARM_MSG_1004285, "%s: zero length", diag);
 		majStat = GSS_S_DEFECTIVE_TOKEN;
 		goto Done;
 	    }
 	} else {
+	    gflog_info(GFARM_MSG_1004286, "%s: gss_wrap fails", diag);
+	    gfarmGssPrintMajorStatus(majStat);
+	    gfarmGssPrintMinorStatus(minStat);
 	    break;
 	}
     } while (rem > 0);
@@ -934,7 +940,7 @@ gfarmGssReceive(int fd, gss_ctx_id_t sCtx, gfarm_int8_t **bufPtr,
 {
     int ret = -1;
     OM_uint32 majStat;
-    OM_uint32 minStat;
+    OM_uint32 minStat, minStat2;
 
     gss_buffer_desc inputToken = GSS_C_EMPTY_BUFFER;
     gss_buffer_t itPtr = &inputToken;
@@ -947,6 +953,7 @@ gfarmGssReceive(int fd, gss_ctx_id_t sCtx, gfarm_int8_t **bufPtr,
     int len;
     gfarm_int8_t *buf = NULL;
     int i;
+    static const char diag[] = "gfarmGssReceive";
 
     /*
      * Receive a length of a PLAIN TEXT.
@@ -963,11 +970,13 @@ gfarmGssReceive(int fd, gss_ctx_id_t sCtx, gfarm_int8_t **bufPtr,
 	majStat = GSS_S_COMPLETE;
 	goto Done;
     } else if (i != 1) {
+	gflog_info(GFARM_MSG_1004287, "%s: %s", diag, strerror(errno));
 	majStat = GSS_S_CALL_INACCESSIBLE_READ;
 	goto Done;
     }
     GFARM_MALLOC_ARRAY(buf, n);
     if (buf == NULL) {
+	gflog_info(GFARM_MSG_1004288, "%s: no memory", diag);
 	majStat = GSS_S_FAILURE;
 	goto Done;
     }
@@ -975,6 +984,7 @@ gfarmGssReceive(int fd, gss_ctx_id_t sCtx, gfarm_int8_t **bufPtr,
     rem = n;
     do {
 	if (gfarmGssReceiveToken(fd, itPtr, timeoutMsec) <= 0) {
+	    gflog_info(GFARM_MSG_1004289, "%s: %s", diag, strerror(errno));
 	    majStat = GSS_S_DEFECTIVE_TOKEN|GSS_S_CALL_INACCESSIBLE_READ;
 	    goto Done;
 	}
@@ -982,7 +992,7 @@ gfarmGssReceive(int fd, gss_ctx_id_t sCtx, gfarm_int8_t **bufPtr,
 			     (const gss_buffer_t)itPtr,
 			     otPtr,
 			     NULL, NULL);
-	gss_release_buffer(&minStat, itPtr);
+	gss_release_buffer(&minStat2, itPtr);
 	if (majStat == GSS_S_COMPLETE) {
 	    if (otPtr->length > 0) {
 		memcpy(buf + sum, otPtr->value, otPtr->length);
@@ -991,10 +1001,14 @@ gfarmGssReceive(int fd, gss_ctx_id_t sCtx, gfarm_int8_t **bufPtr,
 		sum += len;
 		gss_release_buffer(&minStat, otPtr);
 	    } else {
+		gflog_info(GFARM_MSG_1004290, "%s: zero length", diag);
 		majStat = GSS_S_DEFECTIVE_TOKEN;
 		goto Done;
 	    }
 	} else {
+	    gflog_info(GFARM_MSG_1004291, "%s: gss_unwrap fails", diag);
+	    gfarmGssPrintMajorStatus(majStat);
+	    gfarmGssPrintMinorStatus(minStat);
 	    break;
 	}
     } while (rem > 0);
@@ -1283,7 +1297,7 @@ gfarmGssInitiateSecurityContextReceiveToken(int events, int fd,
     } else {
 	assert(events == GFARM_EVENT_READ);
 	tknStat = gfarmGssReceiveToken(fd, state->itPtr,
-				       GFARM_GSS_TIMEOUT_INFINITE);
+			GFARM_GSS_AUTH_TIMEOUT_MSEC);
 	if (tknStat <= 0) {
 	    gflog_auth_error(GFARM_MSG_1000624,
 		"gfarmGssInitiateSecurityContextReceiveToken(): "
