@@ -31,6 +31,7 @@ int gflog_fatal_action_name_to_number(const char *);
 
 /* privlock */
 
+void gfarm_privilege_lock_disable(void);
 void gfarm_privilege_lock(const char *);
 void gfarm_privilege_unlock(const char *);
 
@@ -51,6 +52,7 @@ void gfarm_sleep(long);
 
 #define GFARM_MILLISEC_BY_MICROSEC	1000
 #define GFARM_SECOND_BY_MICROSEC	1000000
+#define GFARM_SECOND_BY_MILLISEC	1000
 
 struct timeval;
 int gfarm_timeval_cmp(const struct timeval *, const struct timeval *);
