@@ -13,7 +13,7 @@
  */
 
 /*
- * $Id: db_ldap.c 7330 2012-12-04 10:22:36Z takuya-i $
+ * $Id: db_ldap.c 9768 2015-06-24 15:21:11Z tatebe $
  */
 
 #include <pthread.h>	/* db_access.h currently needs this */
@@ -2340,6 +2340,47 @@ gfarm_ldap_filecopy_load(
 	    &gfarm_ldap_db_filecopy_ops));
 }
 
+static char ldap_hostname_dn_template[] = "hostname=%s, %s";
+
+static char *
+gfarm_ldap_hostname_make_dn(void *vkey)
+{
+	char *key = vkey;
+	char *dn;
+
+	GFARM_MALLOC_ARRAY(dn, strlen(ldap_hostname_dn_template)
+	    + strlen(key) + strlen(gfarm_ldap_base_dn) + 1);
+	if (dn == NULL) {
+		gflog_debug(GFARM_MSG_1004281,
+			"allocation of string 'dn' failed");
+		return (NULL);
+	}
+	sprintf(dn, gfarm_ldap_db_filecopy_ops.dn_template,
+	    key, gfarm_ldap_base_dn);
+	return (dn);
+}
+
+static const struct gfarm_ldap_generic_info_ops
+    gfarm_ldap_db_filecopy_remove_by_host_ops = {
+	NULL,
+	"(objectclass=GFarmFileCopy)",
+	ldap_hostname_dn_template,
+	gfarm_ldap_hostname_make_dn,
+	NULL,
+};
+
+/* only called at initialization, bypass journal */
+static gfarm_error_t
+gfarm_ldap_filecopy_remove_by_host(gfarm_uint64_t seqnum, char *hostname)
+{
+	gfarm_error_t e;
+
+	e = gfarm_ldap_generic_info_remove(hostname,
+	    &gfarm_ldap_db_filecopy_remove_by_host_ops);
+	free(hostname);
+	return (e);
+}
+
 /**********************************************************************/
 
 static char *gfarm_ldap_db_deadfilecopy_make_dn(void *vkey);
@@ -2474,6 +2515,27 @@ gfarm_ldap_deadfilecopy_load(
 	    &gfarm_ldap_db_deadfilecopy_ops));
 }
 
+static const struct gfarm_ldap_generic_info_ops
+    gfarm_ldap_db_deadfilecopy_remove_by_host_ops = {
+	NULL,
+	"(objectclass=GFarmDeadFileCopy)",
+	ldap_hostname_dn_template,
+	gfarm_ldap_hostname_make_dn,
+	NULL,
+};
+
+/* only called at initialization, bypass journal */
+static gfarm_error_t
+gfarm_ldap_deadfilecopy_remove_by_host(gfarm_uint64_t seqnum, char *hostname)
+{
+	gfarm_error_t e;
+
+	e = gfarm_ldap_generic_info_remove(hostname,
+	    &gfarm_ldap_db_deadfilecopy_remove_by_host_ops);
+	free(hostname);
+	return (e);
+}
+
 /**********************************************************************/
 
 static char *gfarm_ldap_db_direntry_make_dn(void *vkey);
@@ -3181,10 +3243,12 @@ const struct db_ops db_ldap_ops = {
 
 	gfarm_ldap_filecopy_add,
 	gfarm_ldap_filecopy_remove,
+	gfarm_ldap_filecopy_remove_by_host,
 	gfarm_ldap_filecopy_load,
 
 	gfarm_ldap_deadfilecopy_add,
 	gfarm_ldap_deadfilecopy_remove,
+	gfarm_ldap_deadfilecopy_remove_by_host,
 	gfarm_ldap_deadfilecopy_load,
 
 	gfarm_ldap_direntry_add,
