@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 10144 2016-04-25 05:45:06Z tatebe $
+ * $Id: gfmd.c 10230 2016-09-27 19:13:42Z n-soda $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -517,6 +517,9 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_PROCESS_SET:
 		e = gfm_server_process_set(peer, from_client, skip);
 		break;
+	case GFM_PROTO_PROCESS_FD_INFO:
+		e = gfm_server_process_fd_info(peer, from_client, skip);
+		break;
 	case GFJ_PROTO_LOCK_REGISTER:
 		e = gfj_server_lock_register(peer, from_client, skip); break;
 	case GFJ_PROTO_UNLOCK_REGISTER:
@@ -1420,7 +1423,9 @@ sigs_handler(void *p)
 		case SIGXCPU:
 		case SIGXFSZ:
 #ifdef SIGPWR
+#if !defined(SIGINFO) || (SIGPWR != SIGINFO) /* see SF.net #961 */
 		case SIGPWR:
+#endif
 #endif
 			/* terminate gfmd */
 			break;
