@@ -1,5 +1,5 @@
 /*
- * $Id: host.c 9914 2015-10-13 03:38:09Z n-soda $
+ * $Id: host.c 10207 2016-08-24 01:56:26Z tatebe $
  */
 
 #include <assert.h>
@@ -657,9 +657,8 @@ host_is_disk_available(struct host *h, gfarm_off_t size)
 		avail = 0;
 	back_channel_mutex_unlock(h, diag);
 
-	if (minfree < size)
-		minfree = size;
-	return (avail >= minfree);
+	/* to reduce no space risk, keep minimum disk space */
+	return (avail >= minfree + size);
 }
 
 int
