@@ -1,5 +1,5 @@
 /*
- * $Id: quota.h 9872 2015-08-25 06:29:05Z takuya-i $
+ * $Id: quota.h 9969 2015-12-08 08:25:51Z tatebe $
  */
 
 /* related to struct user and struct group */
@@ -37,7 +37,8 @@ void quota_data_init(struct quota *);
 
 struct user;
 struct group;
-gfarm_error_t quota_limit_check(struct user *, struct group *, int, int);
+gfarm_error_t quota_limit_check(struct user *, struct group *, int, int,
+	gfarm_off_t);
 void quota_user_remove(struct user *);
 void quota_group_remove(struct group *);
 
