@@ -4,6 +4,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -100,7 +101,7 @@ move_or_copy_to_lost_found(int op, const char *file, int fd, struct stat *stp,
 	gfarm_error_t e;
 	int save_errno;
 	struct gfarm_timespec mtime;
-	char *newpath, *path = NULL;
+	char *newpath, *newfile, *path = NULL;
 	gfarm_ino_t inum_new;
 	gfarm_uint64_t gen_new;
 	struct stat sb1;
@@ -136,37 +137,43 @@ move_or_copy_to_lost_found(int op, const char *file, int fd, struct stat *stp,
 		 */
 		return (e);
 	}
-	gfsd_local_path(inum_new, gen_new, diag, &newpath);
-	if (gfsd_create_ancestor_dir(newpath)) {
-		save_errno = errno;
-		gflog_error(GFARM_MSG_1003521,
-		    "%s: cannot create ancestor directory: %s",
-		    newpath, strerror(save_errno));
-		free(newpath);
-		return (gfarm_errno_to_error(save_errno));
-	}
 	switch (op) {
 	case 0: /* rename */
 		if (file == NULL) {
-			gfsd_local_path(inum, gen, diag, &path);
+			gfsd_local_path2(inum, gen, diag, &path,
+			    inum_new, gen_new, diag, &newpath);
 			file = path;
+			newfile = newpath;
+		} else {
+			gfsd_local_path(inum_new, gen_new, diag, &newpath);
+			newfile = gfsd_skip_spool_root(newpath);
 		}
-		if (rename(file, newpath)) {
+		if (gfsd_create_ancestor_dir(newfile)) {
+			save_errno = errno;
+			gflog_error(GFARM_MSG_1003521,
+			    "%s: cannot create ancestor directory: %s",
+			    newfile, strerror(save_errno));
+			free(path);
+			free(newpath);
+			return (gfarm_errno_to_error(save_errno));
+		}
+		if (rename(file, newfile)) {
 			save_errno = errno;
 			gflog_error(GFARM_MSG_1003522,
-			    "%s: cannot rename to %s: %s", file, newpath,
+			    "%s: cannot rename to %s: %s", file, newfile,
 			    strerror(save_errno));
 			e = gfarm_errno_to_error(save_errno);
 		}
 		free(path);
 		break;
 	case 1: /* copy */
-		if ((e = gfsd_copy_file(fd, newpath)) != GFARM_ERR_NO_ERROR)
-			gflog_error(GFARM_MSG_1004189,
-			    "inode %lld:%lld: cannot copy to %s, "
+		if ((e = gfsd_copy_file(fd, inum_new, gen_new, diag, &newpath))
+		    != GFARM_ERR_NO_ERROR)
+			gflog_error(GFARM_MSG_1004500,
+			    "inode %lld:%lld: cannot be copied, "
 			    "invalid file may remain: %s",
 			    (unsigned long long)inum, (unsigned long long)gen,
-			    newpath, gfarm_error_string(e));
+			    gfarm_error_string(e));
 		else if (stat(newpath, &sb1) == -1) {
 			save_errno = errno;
 			gflog_error(GFARM_MSG_1004190,
@@ -494,7 +501,8 @@ check_existing(
 	gfarm_ino_t inum, gfarm_uint64_t gen, gfarm_off_t size)
 {
 	gfarm_error_t e;
-	char *path, *file;
+	char *path;
+	const char *file;
 	struct stat st;
 	int save_errno, lost = 0;
 	gfarm_ino_t inum2;
@@ -506,7 +514,7 @@ check_existing(
 	 * all replica-references are deleted....
 	 */
 	gfsd_local_path(inum, gen, "compare_file", &path);
-	file = path + gfarm_spool_root_len + 1;
+	file = gfsd_skip_spool_root(path);
 	get_inum_gen(file, &inum2, &gen2);
 	if (inum != inum2 || gen != gen2)
 		fatal(GFARM_MSG_1003533,
@@ -608,6 +616,7 @@ void
 gfsd_spool_check()
 {
 	struct gfarm_hash_table *hash_ok; /* valid files */
+	int i;
 
 	gflog_debug(GFARM_MSG_1003680, "spool_check_level=%s",
 	    gfarm_spool_check_level_get_by_name());
@@ -628,7 +637,14 @@ gfsd_spool_check()
 	default:
 		return;
 	}
-	(void)check_spool("data", hash_ok);
+	for (i = 0; i < gfarm_spool_root_num; ++i) {
+		if (gfarm_spool_root[i] == NULL)
+			break;
+		if (chdir(gfarm_spool_root[i]) == -1)
+			gflog_fatal_errno(GFARM_MSG_1004484, "chdir(%s)",
+			    gfarm_spool_root[i]);
+		(void)check_spool("data", hash_ok);
+	}
 	if (hash_ok)
 		gfarm_hash_table_free(hash_ok);
 }
